package com.factory.service.impl.ini;

import cn.hutool.core.convert.Convert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.enumPo.CheckStatus;
import com.factory.base.entity.enumPo.CheckTypeSubmit;
import com.factory.base.entity.enumPo.PlanType;
import com.factory.base.entity.enumPo.ReviewStatus;
import com.factory.base.entity.po.benteler.BentelerIniCheckItem;
import com.factory.base.entity.po.benteler.BentelerIniVersion;
import com.factory.base.entity.po.benteler.BentelerIniVersionItem;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.entity.vo.req.BaseIdReq;
import com.factory.base.entity.vo.req.BasePageIdReq;
import com.factory.base.entity.vo.req.BaseTypeReq;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.entity.vo.req.pad.GetVersionCheckItemReq;
import com.factory.entity.vo.req.pad.workDetailsReq;
import com.factory.entity.vo.req.web.*;
import com.factory.entity.vo.res.pad.ini.getNowCheckStatusRes;
import com.factory.entity.vo.res.pad.ini.planlistRes;
import com.factory.entity.vo.res.pad.ini.workDetailsRes;
import com.factory.entity.vo.res.pad.ini.workTypeDetailsRes;
import com.factory.entity.vo.res.web.ini.GetVersionBaseRes;
import com.factory.entity.vo.res.web.ini.GetVersionHistoryRes;
import com.factory.entity.vo.res.web.ini.PagePlanRes;
import com.factory.entity.vo.res.web.ini.unAuditListRes;
import com.factory.mapper.ini.BentelerIniVersionMapper;
import com.factory.service.ini.BentelerIniCheckResultService;
import com.factory.service.ini.BentelerIniVersionItemService;
import com.factory.service.ini.BentelerIniVersionService;
import com.factory.service.BentelerPlanService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;


@Service("bentelerIniVersionService")
@Transactional(rollbackFor = Exception.class)
public class BentelerIniVersionServiceImpl extends ServiceImpl<BentelerIniVersionMapper, BentelerIniVersion> implements BentelerIniVersionService {

    @Autowired
    private BentelerIniVersionMapper versionMapper;
    @Autowired
    private BentelerPlanService planService;
    @Autowired
    private BentelerIniVersionService versionService;
    @Autowired
    private BentelerIniVersionItemService versionItemService;
    @Autowired
    private BentelerIniCheckResultService checkResultService;

    @Override
    public ResponseBean<PageBean<PagePlanRes>> planlist(PlanListReq req) {
        String name = req.getName();
        Integer checkStatus = req.getCheckStatus();
        Page<PagePlanRes> page = new Page<>(req.getPage(), req.getSize());
        return ResponseBeanBuilder.ok(versionMapper.planlist(page, name, checkStatus));

    }

    @Override
    public ResponseBean addIniPlan(AddIniPlanReq req) {
        //校验
        if (planNameRepetition(req.getName())) {
            return ResponseBeanBuilder.fail("计划名称重复");
        }
        //新增计划
        BentelerPlan plan = BentelerPlan.builder()
                .planType(PlanType.首件.value)
                .name(req.getName())
                .pauseFlag(false)
                .build();
        planService.save(plan);
        //新增版本
        ArrayList<Integer> allCheckType = req.getVersionWorks().stream().collect(
                ArrayList::new,
                (theList, item) -> {
                    Collection<Integer> union1 = CollectionUtils.union(theList,
                            item.getBentelerIniVersionItems().stream().collect(
                                    ArrayList::new,
                                    (theList2, item2) -> {
                                        Collection<Integer> union = CollectionUtils.union(theList2, item2.getCheckType());
                                        theList2.addAll(union);
                                    },
                                    ArrayList::addAll
                            )
                    );
                    theList.addAll(union1);
                },
                ArrayList::addAll

        );
        Map<String, String> baseMap = new HashMap<String, String>() {{
            put("project", req.getName());
            put("part", req.getName());
            put("btlPartNo", req.getName());
            put("clientPartNo", req.getName());
            put("verson", req.getName());
            put("Dmba", req.getName());
        }};

        List<Long> principalIds = req.getPrincipalIds();
        Long principleOne = principalIds.get(0);
        Long principleTwo = null;
        if (principalIds.size() > 1) {
            principleTwo = principalIds.get(1);
        }
        BentelerIniVersion version = BentelerIniVersion.builder()
                .planId(plan.getId())
                .allChecktype(Convert.toStr(allCheckType))
                .baseMap(Convert.toStr(baseMap))
                .checkStatus(CheckStatus.未完成.value)
                .beginDate(req.getBeginDate())
                .endDate(req.getEndDate())
                .guideDocumentId(req.getGuideDocumentId())
                .principleOne(principleOne)
                .principleTwo(principleTwo)
                .reviewStatus(ReviewStatus.待质量经理审核.value)

                .checkBegintime(req.getCheckBegintime())
                .checkEndtime(req.getCheckEndtime())
                .remindIniPad(req.getRemindIniPad())
                .remindTwoPad(req.getRemindTwoPad())
                .upgradeTime(req.getUpgradeTime())
                .build();
        versionService.save(version);
        //新增检查项
        ArrayList<BentelerIniVersionItem> VersionItems = new ArrayList<>();
        req.getVersionWorks().forEach(versionWork -> {
            versionWork.getBentelerIniVersionItems().forEach(item -> {
                BentelerIniVersionItem versionItem = BentelerIniVersionItem.builder()
                        .planId(plan.getId())
                        .versionId(version.getId())
                        .workstation(versionWork.getWorkstation())
                        .productionLine(versionWork.getProductionLine())
                        .checkStandardTop(item.getCheckStandardTop())
                        .checkStandardFloor(item.getCheckStandardFloor())
                        .checkType(Convert.toStr(item.getCheckType()))
                        .dataType(item.getDataType())
                        .frequency(item.getFrequency())
                        .frequencyNo(item.getFrequencyNo())
                        .inspectionItem(item.getInspectionItem())
                        .measuRemark(item.getMeasuRemark())
                        .level(item.getLevel())
                        .method(item.getMethod())
                        .unit(item.getUnit())
                        .tool(item.getTool())
                        .toolRemark(item.getToolRemark())
                        .workPrefix(item.getWorkPrefix())
                        .workStationBegin(item.getWorkStationBegin())
                        .workStationEnd(item.getWorkStationEnd())

                        .checkBegintime(req.getCheckBegintime())
                        .checkEndtime(req.getCheckEndtime())
                        .remindIniPad(req.getRemindIniPad())
                        .remindTwoPad(req.getRemindTwoPad())
                        .upgradeTime(req.getUpgradeTime())
                        .build();
                VersionItems.add(versionItem);
            });

        });
        versionItemService.saveBatch(VersionItems);


        return ResponseBeanBuilder.ok();
    }

    @Override
    public ResponseBean disablePlan(BooleanReq req) {
        Long versionId = req.getId();

        Long planId = versionService.getById(versionId).getPlanId();

        planService.updateById(BentelerPlan.builder()
                .id(planId)
                .pauseFlag(req.getBoolea())
                .build());
        return ResponseBeanBuilder.ok();
    }

    @Override
    public ResponseBean<PageBean<unAuditListRes>> unAuditList(PlanListReq req) {
        Page<unAuditListRes> unAuditListResPage = versionMapper.unAuditList(req);
        return ResponseBeanBuilder.ok();
    }

    @Override
    public ResponseBean<GetVersionBaseRes> getVersionBase(BaseIdReq req) {
        Long versionId = req.getId();
        BentelerIniVersion version = versionService.getById(versionId);
        String baseMap = version.getBaseMap();
        Map<String, String> stringStringMap = Convert.toMap(String.class, String.class, baseMap);
        GetVersionBaseRes basemap = GetVersionBaseRes.builder()
                .project(stringStringMap.get("project"))
                .part(stringStringMap.get("part"))
                .btlPartNo(stringStringMap.get("btlPartNo"))
                .clientPartNo(stringStringMap.get("clientPartNo"))
                .verson(stringStringMap.get("verson"))
                .Dmba(stringStringMap.get("Dmba"))
                .build();
        return ResponseBeanBuilder.ok(basemap);
    }


    @Override
    public ResponseBean<AddIniPlanReq> getUnAuditDetails(BaseIdReq req) {
        Long planId = req.getId();
        BentelerPlan plan_DB = planService.getById(planId);
        BentelerIniVersion unAuditVersion = versionMapper.getUnAuditVersion(planId);
        BentelerPlan plan = BentelerPlan.builder()
                .name(plan_DB.getName())
                .version(unAuditVersion)
                .build();
        return ResponseBeanBuilder.ok(planPo2planReq(plan));
    }

    @Override
    public ResponseBean<AddIniPlanReq> getCurrentDetails(BaseIdReq req) {
        Long versionId = req.getId();
        BentelerIniVersion version = versionService.getById(versionId);
        BentelerPlan plan = planService.getById(version.getPlanId());
        plan.setVersion(versionMapper.getUnAuditVersion(plan.getId()));
        return ResponseBeanBuilder.ok(planPo2planReq(plan));
    }

    @Override
    public ResponseBean addIniVersion(AddIniPlanReq req) {
        Long versionId = req.getVersionId();
        Long planId = versionService.getById(versionId).getPlanId();
        String name = planService.getById(planId).getName();
        if (!req.getName().equals(name)) {
            if (planNameRepetition(req.getName())) {
                return ResponseBeanBuilder.fail("计划名称重复");
            }
        }
        //转换数据库
        BentelerPlan plan_req = planReq2planPo(req, ReviewStatus.待质量经理审核);
        BentelerIniVersion version_req = plan_req.getVersion();
        version_req.setPlanId(planId);
        versionService.save(version_req);
        List<BentelerIniVersionItem> versionItem_req = version_req.getVersionItem();
        versionItem_req.forEach(item -> {
            item.setPlanId(planId);
            item.setVersionId(version_req.getId());
        });
        versionItemService.saveBatch(versionItem_req);
        //获取待审核版本  并删除
        BentelerIniVersion unAuditVersion = versionMapper.getUnAuditVersion(planId);
        if (unAuditVersion != null) {
            Long versionId_DB = unAuditVersion.getId();
            versionService.removeById(versionId_DB);

        }


        return ResponseBeanBuilder.ok();
    }


    @Override
    public ResponseBean<Boolean> getIsUnAuditVersionId(BaseIdReq req) {
        //查询是否存在其他待审核
        Long versionId = req.getId();
        BentelerIniVersion version_DB = versionService.getById(versionId);
        BentelerIniVersion unAuditVersion = versionMapper.getWaitAuditVersion(version_DB.getPlanId());
        if (unAuditVersion != null) {
            return ResponseBeanBuilder.ok(true);
        }
        return ResponseBeanBuilder.ok(false);
    }

    @Override
    public ResponseBean<PageBean<GetVersionHistoryRes>> historyVersion(BasePageIdReq req) {
        Long vertsionId = req.getId();
        BentelerIniVersion version = versionService.getById(vertsionId);
        Long planId = version.getPlanId();
        Page<BentelerIniVersion> page = new Page<>(req.getPage(), req.getSize());
        Page<BentelerIniVersion> versionPage = versionService.page(page, Wrappers.<BentelerIniVersion>lambdaQuery()
                .eq(BentelerIniVersion::getPlanId, planId)
                .eq(BentelerIniVersion::getReviewStatus, ReviewStatus.历史版本.value)
        );

        ResponseBean<PageBean<GetVersionHistoryRes>> ok = ResponseBeanBuilder.ok(versionPage,
                item -> GetVersionHistoryRes.builder()
                        .id(item.getId())
                        .createdTime(item.getCreatedTime())
                        .planId(item.getPlanId())
                        .build()
        );
        return ok;
    }

    @Override
    public ResponseBean<planlistRes> planPadlist() {
        List<getNowCheckStatusRes> inifinish = versionMapper.getNowCheckStatus(CheckStatus.首件已完成.value);
        List<getNowCheckStatusRes> twoFinish = versionMapper.getNowCheckStatus(CheckStatus.两百已完成.value);
        List<getNowCheckStatusRes> unFinish = versionMapper.getNowCheckStatus(CheckStatus.未完成.value);


         planlistRes res = planlistRes.builder()
                .inifinish(inifinish)
                .twoFinish(twoFinish)
                .unFinish(unFinish)
                .build();
        return ResponseBeanBuilder.ok(res);
    }

    @Override
    public ResponseBean<List<workDetailsRes>> workDetails(BaseTypeReq req) {
        Long versionId = req.getId();
        Integer type = req.getType();
        GetVersionCheckItemReq getVersionCheckItemReq = GetVersionCheckItemReq.builder()
                .versionId(versionId)
                .checkType(type)
                .localDate(LocalDate.now())
                .build();
        List<BentelerIniCheckItem> checkItems = versionMapper.getVersionCheckItem( getVersionCheckItemReq);
        Map<Long, List<BentelerIniCheckItem>> versionItemId_checkItem_map =
                checkItems.stream().collect(Collectors.groupingBy(BentelerIniCheckItem::getVersionItemId));

        List<workDetailsRes> resList = new ArrayList<>();

        versionItemId_checkItem_map.entrySet().stream().forEach(item->{
            BentelerIniVersionItem versionitem = versionItemService.getById(item.getKey());
            boolean allSaveOrSubmit = item.getValue().stream().allMatch(checkItem -> {
                Integer submitType = checkItem.getSubmitType();
                return submitType.equals(CheckTypeSubmit.保存.value)||submitType.equals(CheckTypeSubmit.提交.value);
            });
            Integer submitType =null;
            if (allSaveOrSubmit){
                submitType = 1;
            }else {
                submitType = 2;
            }
            workDetailsRes res = workDetailsRes.builder()
                    .workName(versionitem.getWorkstation()+"-"+versionitem.getProductionLine())
                    .submitType(submitType)
                    .build();
            resList.add(res);
        });
        return ResponseBeanBuilder.ok(resList);
    }

    @Override
    public ResponseBean<workTypeDetailsRes> workTypeDetails(workDetailsReq req) {

        return null;
    }

    //PlanReq转化planPo
    public BentelerPlan planReq2planPo(AddIniPlanReq req, ReviewStatus reviewStatus) {
        //计划
        BentelerPlan plan = BentelerPlan.builder()
                .planType(PlanType.首件.value)
                .name(req.getName())
                .pauseFlag(false)
                .build();
        //version
        ArrayList<Integer> allCheckType = req.getVersionWorks().stream().collect(
                ArrayList::new,
                (theList, item) -> CollectionUtils.union(theList, item.getBentelerIniVersionItems().stream().collect(
                        ArrayList::new,
                        (theList2, item2) -> CollectionUtils.union(theList2, item2.getCheckType()),
                        ArrayList::addAll
                )),
                ArrayList::addAll

        );
        Map<String, String> baseMap = new HashMap<String, String>() {{
            put("project", req.getName());
            put("part", req.getName());
            put("btlPartNo", req.getName());
            put("clientPartNo", req.getName());
            put("verson", req.getName());
            put("Dmba", req.getName());
        }};

        List<Long> principalIds = req.getPrincipalIds();
        Long principleOne = principalIds.get(0);
        Long principleTwo = null;
        if (principalIds.size() > 1) {
            principleTwo = principalIds.get(1);
        }
        BentelerIniVersion version = BentelerIniVersion.builder()
                .planId(plan.getId())
                .allChecktype(Convert.toStr(allCheckType))
                .baseMap(Convert.toStr(baseMap))
                .checkStatus(CheckStatus.未完成.value)
                .beginDate(req.getBeginDate())
                .endDate(req.getEndDate())
                .guideDocumentId(req.getGuideDocumentId())
                .principleOne(principleOne)
                .principleTwo(principleTwo)
                .reviewStatus(reviewStatus.value)
                .build();
        //item
        ArrayList<BentelerIniVersionItem> VersionItems = new ArrayList<>();
        req.getVersionWorks().forEach(versionWork -> {
            versionWork.getBentelerIniVersionItems().forEach(item -> {
                BentelerIniVersionItem versionItem = BentelerIniVersionItem.builder()
                        .planId(plan.getId())
                        .versionId(version.getId())
                        .workstation(versionWork.getWorkstation())
                        .productionLine(versionWork.getProductionLine())
                        .checkStandardTop(item.getCheckStandardTop())
                        .checkStandardFloor(item.getCheckStandardFloor())
                        .checkType(Convert.toStr(item.getCheckType()))
                        .dataType(item.getDataType())
                        .frequency(item.getFrequency())
                        .frequencyNo(item.getFrequencyNo())
                        .inspectionItem(item.getInspectionItem())
                        .measuRemark(item.getMeasuRemark())
                        .level(item.getLevel())
                        .method(item.getMethod())
                        .unit(item.getUnit())
                        .tool(item.getTool())
                        .toolRemark(item.getToolRemark())
                        .workPrefix(item.getWorkPrefix())
                        .workStationBegin(item.getWorkStationBegin())
                        .workStationEnd(item.getWorkStationEnd())

                        .checkBegintime(req.getCheckBegintime())
                        .checkEndtime(req.getCheckEndtime())
                        .remindIniPad(req.getRemindIniPad())
                        .remindTwoPad(req.getRemindTwoPad())
                        .upgradeTime(req.getUpgradeTime())
                        .build();
                VersionItems.add(versionItem);
            });

        });
        version.setVersionItem(VersionItems);
        plan.setVersion(version);
        return plan;
    }

    //planPo转化AddIniPlanReq
    public AddIniPlanReq planPo2planReq(BentelerPlan plan) {
        BentelerIniVersion version = plan.getVersion();
        List<BentelerIniVersionItem> versionItem = version.getVersionItem();
        Map<String, List<BentelerIniVersionItem>> workGroup =
                versionItem.stream().collect(
                        Collectors.groupingBy(item -> item.getWorkstation() + "_" + item.getProductionLine()
                        )
                );
        List<AddIniPlanWorkReq> works = workGroup.entrySet().stream().map(item -> {
            BentelerIniVersionItem bentelerIniVersionItem = item.getValue().stream().findFirst().orElse(null);
            List<AddIniPlanItemReq> itemReqs = item.getValue().stream().map(versionItemf -> {
                List<Integer> checkTypes = Convert.toList(Integer.class, versionItemf.getCheckType());
                return AddIniPlanItemReq.builder()
                        .checkType(checkTypes)
                        .measuRemark(versionItemf.getMeasuRemark())
                        .method(versionItemf.getMethod())
                        .checkStandardTop(versionItemf.getCheckStandardTop())
                        .checkStandardFloor(versionItemf.getCheckStandardFloor())
                        .workPrefix(versionItemf.getWorkPrefix())
                        .workStationBegin(versionItemf.getWorkStationBegin())
                        .workStationEnd(versionItemf.getWorkStationEnd())
                        .frequency(versionItemf.getFrequency())
                        .frequencyNo(versionItemf.getFrequencyNo())
                        .dataType(versionItemf.getDataType())
                        .level(versionItemf.getLevel())
                        .unit(versionItemf.getUnit())
                        .tool(versionItemf.getTool())
                        .toolRemark(versionItemf.getToolRemark())
                        .measuRemark(versionItemf.getMeasuRemark())
                        .inspectionItem(versionItemf.getInspectionItem())
                        .build();
            }).collect(Collectors.toList());


            return AddIniPlanWorkReq.builder()
                    .productionLine(bentelerIniVersionItem.getProductionLine())
                    .workstation(bentelerIniVersionItem.getWorkstation())
                    .BentelerIniVersionItems(itemReqs)
                    .build();
        }).collect(Collectors.toList());

        String baseMapStr = version.getBaseMap();
        Map<String, String> baseMap = Convert.toMap(String.class, String.class, baseMapStr);
        //责任人
        Long principleOne = version.getPrincipleOne();
        Long getPrincipleTwo = version.getPrincipleTwo();

        return AddIniPlanReq.builder()
                .name(plan.getName())
                .principalIds(Arrays.asList(principleOne, getPrincipleTwo))
                .checkEndtime(version.getCheckEndtime())
                .remindIniPad(version.getRemindIniPad())
                .remindTwoPad(version.getRemindTwoPad())
                .upgradeTime(version.getUpgradeTime())
                .beginDate(version.getBeginDate())
                .endDate(version.getEndDate())
                .guideDocumentId(version.getGuideDocumentId())
                .project(baseMap.get("project"))
                .part(baseMap.get("part"))
                .btlPartNo(baseMap.get("btlPartNo"))
                .clientPartNo(baseMap.get("clientPartNo"))
                .verson(baseMap.get("verson"))
                .Dmba(baseMap.get("Dmba"))

                .checkBegintime(version.getCheckBegintime())
                .checkEndtime(version.getCheckEndtime())
                .remindIniPad(version.getRemindIniPad())
                .remindTwoPad(version.getRemindTwoPad())
                .upgradeTime(version.getUpgradeTime())

                .versionWorks(works)
                .build();
    }

    public boolean planNameRepetition(String name) {
        List<BentelerPlan> plans = planService.list(Wrappers.<BentelerPlan>lambdaQuery()
                .eq(BentelerPlan::getName, name));
        if (CollectionUtils.isNotEmpty(plans)) {
            return true;
        }
        return false;
    }
}